<?php

namespace Drupal\paragraphs_collection;

/**
 * Provides discovery for a YAML style files in specific directories.
 *
 * @package Drupal\paragraphs_collection
 */
interface StyleDiscoveryInterface {

  /**
   * Gets libraries for a specific style or empty list if style is not found.
   *
   * @param $style
   *   The name of the style.
   * @return array
   *   The names of the libraries, or empty list if not found.
   */
  public function getLibraries($style);

  /**
   * Gets sorted style titles keyed by their names belonging to the given group.
   *
   * If an empty string is given, returns all styles.
   *
   * @param string $group
   *   (optional) The style group. Defaults to empty string.
   *
   * @return array
   *   An array of style titles keyed by the respective style machine names.
   */
  public function getStyleOptions($group = '');

  /**
   * Gets style groups.
   *
   * @return array
   *    Collection of style groups.
   */
  public function getStyleGroups();

  /**
   * Returns associative array of name and definition of style.
   *
   * @return array
   *   Collection of styles.
   */
  public function getStyles();

  /**
   * Get style by name.
   *
   * @param string $style
   *   The style name.
   * @param string|null $default
   *   (optional) The default style if the specified style does not exist.
   *
   * @return array|null
   *   The style configuration array of the given style or NULL.
   */
  public function getStyle($style, $default = NULL);

  /**
   * Gets group label.
   *
   * @param string $group_id
   *   The group id.
   *
   * @return string
   *   The translatable group label.
   */
  public function getGroupLabel($group_id);

  /**
   * Reset the cached definitions.
   */
  public function reset();

}
